0010***********************************************************************
0020* PROGRAM-ID.        EXL101RA
0030* PROGRAM NAME.      Interface to Excel & WorkSheets
0040* PROGRAMMER NAME.   C. Scott
0050* ANALYST NAME.      B. Engele
0060*
0070*    *** SPECIAL PROCEDURES ***
0080*        Source should only exist in the PC DBA-EXL1 library, the
0090*        object should be copied to PC SYSTEM FUSR.
0100*
0110*
0120*    *** STATUS LOG ***
0130*  ---------------------------------------------------------------------
0140*  DD/MMM/CCYY | CD | PROGRAMMER | DESCRIPTION OF CHANGES
0150*  ------------+----+------------+--------------------------------------
0160*  19/Aug/2005 | AA | C. Scott   | New External SubRoutine
0170*  ---------------------------------------------------------------------
0180*
0190*
0200* @@@ START SSSAPP ODM SECTIONS
0210*
0220*    *** PROGRAM NARRATIVE ***
0230*    Interface use the MS Excel application and various functions
0240*    for WorkSheets (pages of a workbook).
0250*    This interface can be used to:
0260*      1. Add a sheet to the active workbook.
0270*      2. Delete a sheet from the active workbook.
0280*      3. Move a sheet within the active workbook.
0290*      4. Select a specific sheet within the active workbook.
0300*      5. Rename (update) sheet names within the active workbook.
0310*
0320*  THIS PROGRAM IS CALLED FROM     Various including RPC
0330*
0340*  THIS PROGRAM EXITS TO PROGRAM   Calling routine
0350*      ON    CONDITION
0360*  OR TO                           SSAAPPIV
0370*      ON    CONDITION
0380*
0390*    *** CONTROL STATEMENTS ***
0400*
0410*  --------------------------------------------------------------------
0420*  | COLUMNS |            |        |     |     |
0430*  | FR | TO | FIELD NAME | SOURCE | LNG | TYP | COMMENTS
0440*  -----+----+------------+--------+-----+-----+-----------------------
0450*  |    |    |            |        |     |     |
0460*  --------------------------------------------------------------------
0470*
0480*    *** FORMS ***
0490*
0500*  --------------------------------------------------------------------
0510*  |   FORM   | RPT # |  DESCRIPTION
0520*  -----------+-------+------------------------------------------------
0530*  | SSSAPPF0 |  9999 |
0540*  --------------------------------------------------------------------
0550*
0560* @@@ END ODM SECTIONS
0570***********************************************************************
0580/*Struct Off
0590Define Data
0600  Parameter Using EXL-OBJ1
0610  Parameter Using EXL100P1
0620  Parameter Using EXL101P1
0630  Local Using EXL101L1
0640End-Define
0650/*Struct On
0660*
0670*********************************
0680Define Subroutine Excel-WorkSheet
0690*********************************
0700Reset #EXL-Error-No #EXL-Error-Msg
0710If #EXL-Application = NULL-HANDLE or #EXL-WorkBook = NULL-HANDLE
0720   or #EXL-Sheets = NULL-HANDLE
0730** *  It's an error at this point, there needs to be an open workbook  * **
0740** *  before any of the worksheet functions can be performed.          * **
0750  #EXL-Error-No := 1
0760  #EXL-Error-Msg := 'EXL101 - There must be an open workbook before using this routine!'
0770  Escape Routine
0780End-If
0790*
0800#WorkBook-Command := *Translate(#EXL-Command, Upper)
0810Decide on First Value of #WorkBook-Command
0820  Value 'ADD'
0830     If #EXL101-Sheet-Num LE 0
0840        #EXL-Error-No := 2
0850        Compress 'EXL101 - Parameter #EXL101-Sheet-Num must be greater than 0 for the'
0860           #EXL-Command 'function.' Into #EXL-Error-Msg
0870        Escape Routine
0880     End-If
0890     Perform Add-Excel-WorkSheet
0900  Value 'DELETE'
0910     Perform Delete-Excel-WorkSheet
0920  Value 'SELECT'
0930     If #EXL101-Sheet-Name Specified And #EXL101-Sheet-Name NE ' '
0940        If #EXL101-Sheet-Num NE 0
0950           #EXL-Error-No := 3
0960           Compress 'EXL101 - Specify #EXL101-Sheet-Num or #EXL101-Sheet-Name,'
0970              'not both for the' #EXL-Command 'function.'
0980              Into #EXL-Error-Msg
0990           Escape Routine
1000        End-If
1010        Examine #EXL100-SheetNames(1:#EXL100-NumSheets) For #EXL101-Sheet-Name
1020           Giving Index In #EXL101-Sheet-Num
1030        If #EXL101-Sheet-Num = 0
1040           #EXL-Error-No := 3
1050           Compress 'EXL101 - Sheet named' #EXL101-Sheet-Name 'not found'
1060              'in the current workbook for the' #EXL-Command 'function.'
1070              Into #EXL-Error-Msg
1080           Escape Routine
1090        End-If
1100     End-If
1110     If #EXL101-Sheet-Num LE 0 OR #EXL101-Sheet-Num GT #EXL100-NumSheets
1120        #EXL-Error-No := 3
1130        Compress 'EXL101 - Parameter #EXL101-Sheet-Num must be between 0'
1140           'and the number of sheets for the'
1150           #EXL-Command 'function.' Into #EXL-Error-Msg
1160        Escape Routine
1170     End-If
1180     Perform Select-Excel-WorkSheet
1190     Send "Activate" To #EXL-Sheet Giving #EXL-Error-No
1200  Value 'MOVE'
1210     If #EXL101-Sheet-Num LT 0 OR #EXL101-Sheet-Num GT #EXL100-NumSheets
1220        #EXL-Error-No := 4
1230        Compress 'EXL101 - Parameter #EXL101-Sheet-Num must be GE 0 and'
1240           'LE the number of sheets for the'
1250           #EXL-Command 'function.' Into #EXL-Error-Msg
1260        Escape Routine
1270     End-If
1280     Perform Move-Excel-WorkSheet
1290  Value 'RENAME'
1300     Perform Update-WorkBook-Sheets
1310  Value 'AUTOFIT'
1320     If #EXL-Sheet = NULL-HANDLE
1330        #EXL-Error-No := 5
1340        Compress 'EXL101 - A sheet must already have been selected before using the'
1350           #EXL-Command 'function.' Into #EXL-Error-Msg
1360        Escape Routine
1370     Else
1380        Send "Columns" To #EXL-Sheet Return #EXL-Sheet-Cells
1390           Giving #EXL-Error-No
1400        If #EXL-Error-No = 0
1410           Send "AutoFit" To #EXL-Sheet-Cells Giving #EXL-Error-No
1420           If #EXL-Error-No NE 0
1430              Compress 'EXL101 - Error' #EXL-Error-No 'from sheet for'
1440              #EXL-Command 'function.' into #EXL-Error-Msg
1450              Escape Routine
1460           End-If
1470        Else
1480           Compress 'EXL101 - Error' #EXL-Error-No 'from Cell selection for'
1490              #EXL-Command 'function.' into #EXL-Error-Msg
1500           Escape Routine
1510        End-If
1520     End-If
1530  None Value
1540     #EXL-Error-No := 99
1550     Compress 'EXL101 - Invalid command:' #EXL-Command 'used.'
1560        Into #EXL-Error-Msg
1570     Escape Routine
1580End-Decide
1590Reset #EXL101-Sheet-Num #EXL-Command
1600*
1610*************************************
1620Define Subroutine Add-Excel-WorkSheet
1630*************************************
1640** *  Add to end so get handle to last sheet  * **
1650  #Temp-NumSheets := #EXL-Sheets.COUNT
1660  Send "Item" To #EXL-Sheets With #Temp-NumSheets
1670     Return #EXL-Sheet Giving #EXL-Error-No
1680  Send "Add" To #EXL-Sheets With 1X #EXL-Sheet #EXL101-Sheet-Num
1690     Return #EXL-Sheet Giving #EXL-Error-No
1700  Perform Update-WorkBook-Sheets
1710End-Subroutine /* Add-WorkBook-Sheets
1720*************************************
1730*
1740****************************************
1750Define Subroutine Delete-Excel-WorkSheet
1760****************************************
1770** *  Remove current active sheet  * **
1780  Send "Delete" TO #EXL-Sheet Giving #EXL-Error-No
1790  If #EXL-Error-No = 0
1800     Perform Update-WorkBook-Sheets
1810  Else
1820     Compress 'EXL101 - Error' #EXL-Error-No 'from delete of active worksheet'
1830        into #EXL-Error-Msg
1840     Escape Module
1850  End-If
1860End-Subroutine /* Delete-Excel-WorkSheet
1870****************************************
1880*
1890**************************************
1900Define Subroutine Move-Excel-WorkSheet
1910**************************************
1920** *  Move the current active sheet after the indexed sheet  * **
1930** *  specified by the #EXL101-Sheet-Num paramter.           * **
1940** *  If the specified sheet is 0 then move the current      * **
1950** *  sheet to the front of the workbook.                    * **
1960  #Temp-Sheet := #EXL-Sheet
1970  If #EXL101-Sheet-Num GT 0  /* Move after sheet index specified
1980     Perform Select-Excel-WorkSheet
1990     If #Temp-Sheet NE #EXL-Sheet
2000        Send "Move" TO #Temp-Sheet With 1X #EXL-Sheet Giving #EXL-Error-No
2010     Else
2020        #EXL-Error-No := 5
2030        #EXL-Error-Msg := 'EXL101 - Sheet already in position specified!'
2040        Escape Module
2050     End-If
2060  Else  /* Move to front (before sheet 1)
2070     #EXL101-Sheet-Num := 1
2080     Perform Select-Excel-WorkSheet
2090     If #Temp-Sheet NE #EXL-Sheet
2100        Send "Move" TO #Temp-Sheet With #EXL-Sheet Giving #EXL-Error-No
2110     Else
2120        #EXL-Error-No := 5
2130        #EXL-Error-Msg := 'EXL101 - Sheet already in position specified!'
2140        Escape Module
2150     End-If
2160  End-If
2170  If #EXL-Error-No NE 0
2180     #EXL-Sheet := #Temp-Sheet
2190     Compress 'EXL101 - Error' #EXL-Error-No
2200        'returned from worksheet move' Into #EXL-Error-Msg
2210     Escape Module
2220  End-IF
2230  Perform Update-Returned-Names
2240  #EXL-Sheet := #Temp-Sheet
2250End-Subroutine /* Move-Excel-WorkSheet
2260**************************************
2270*
2280****************************************
2290Define Subroutine Select-Excel-WorkSheet
2300****************************************
2310  Send "Item" To #EXL-Sheets With #EXL101-Sheet-Num
2320     Return #EXL-Sheet Giving #EXL-Error-No
2330  If #EXL-Error-No NE 0
2340     Compress 'EXL101 - Error' #EXL-Error-No
2350        'returned from worksheet selection' Into #EXL-Error-Msg
2360     Escape Module
2370  End-If
2380End-Subroutine /* Select-Excel-WorkSheet
2390****************************************
2400*
2410*************************************************************
2420*************************************************************
2430** *  Other common code to be used by this routine only  * **
2440*************************************************************
2450*
2460****************************************
2470Define Subroutine Update-WorkBook-Sheets
2480****************************************
2490** *  Update the number and names of sheets in the current workbook  * **
2500  #EXL100-NumSheets := #EXL-Sheets.COUNT
2510  If #EXL100-NumSheets GT 0
2520     Resize Array #EXL100-SheetNames To (*:#EXL100-NumSheets)
2530     For #Loop = 1 to #EXL100-NumSheets
2540        Send "Item" To #EXL-Sheets With #LOOP
2550           Return #EXL-Sheet Giving #EXL-Error-No
2560        If #EXL-Error-No ne 0
2570           Compress 'EXL101 - Error:' #EXL-Error-No 'from getting sheet:' #Loop
2580              Into #EXL-Error-Msg
2590           Escape Module
2600        End-if
2610        If #EXL100-SheetNames(#Loop) NE ' '
2620           #Temp-SheetName := #EXL-Sheet.NAME
2630           If #Temp-SheetName NE #EXL100-SheetNames(#Loop)
2640              #Temp-SheetName := #EXL100-SheetNames(#Loop)
2650              #EXL-Sheet.NAME := #Temp-SheetName
2660           End-If
2670        Else
2680           #EXL100-SheetNames(#Loop) := #EXL-Sheet.NAME
2690        End-If
2700     End-For
2710  End-If
2720End-Subroutine /* Update-WorkBook-Sheets
2730****************************************
2740*
2750***************************************
2760Define Subroutine Update-Returned-Names
2770***************************************
2780** *  Update the names of sheets used in the current workbook  * **
2790  #EXL100-NumSheets := #EXL-Sheets.COUNT
2800  If #EXL100-NumSheets GT 0
2810     For #Loop = 1 to #EXL100-NumSheets
2820        Send "Item" To #EXL-Sheets With #LOOP
2830           Return #EXL-Sheet Giving #EXL-Error-No
2840        If #EXL-Error-No ne 0
2850           Compress 'EXL101 - Error:' #EXL-Error-No 'from getting sheet:' #Loop
2860              Into #EXL-Error-Msg
2870           Escape Routine
2880        End-if
2890        #EXL100-SheetNames(#Loop) := #EXL-Sheet.NAME
2900     End-For
2910  End-If
2920End-Subroutine /* Update-Returned-Names
2930***************************************
2940*
2950**************************************************************
2960** *  End of common code to be used by this routine only  * **
2970**************************************************************
2980**************************************************************
2990*
3000End-Subroutine /* Excel-WorkSheet
3010*********************************
3020*
3030End
